﻿// ==UserScript==
// @name        NPC Robot Repair
// @namespace   http://userscripts.xcom-alliance.info/
// @description Automatically calculate and then use robots required to repair your ship when in combat with NPCs
// @author      Miche (Orion) / Sparkle (Artemis)
// @version     1.1
// @include     http*://*.pardus.at/overview_ship.php
// @include     http*://*.pardus.at/ship2opponent_combat.php?opponentid=*
// @updateURL 	http://userscripts.xcom-alliance.info/npc_robot_repair/pardus_npc_robot_repair.meta.js
// @downloadURL http://userscripts.xcom-alliance.info/npc_robot_repair/pardus_npc_robot_repair.user.js
// @icon 		http://userscripts.xcom-alliance.info/npc_robot_repair/icon.png
// @grant       GM_getValue
// @grant       GM_setValue
// @grant       GM_xmlhttpRequest
// ==/UserScript==

/*****************************************************************************************
	Version Information

	28/06/2013 (Version 1.0)
	  - Initial release for general use

	28/06/2013 (Version 1.1)
	  - Added settings options to NPC combat screen
	  - Added the ability to set a minimum threshold to repair to

*****************************************************************************************/

/*****************************************************************************************
	Get settings
*****************************************************************************************/

function getUniverse() {
	var universe = window.location.host.substr(0, window.location.host.indexOf('.'));
	return universe.charAt(0).toUpperCase() + universe.substring(1).toLowerCase();
};

function getRobotUseShortcut() {
	return GM_getValue(getUniverse() + '_Shortcut_Key', 'b');
};

function getRobotRepairType() {
	var value = GM_getValue(getUniverse() + '_Robot_Repair_Type', '1');
	return parseInt(value, 10);
};

function getRobotRepairMinimum() {
	var value = GM_getValue(getUniverse() + '_Robot_Repair_Min', '230');
	return parseInt(value, 10);
};

/*****************************************************************************************
	Core functionality methods
*****************************************************************************************/

function setArmorStrength() {
	if (typeof GM_xmlhttpRequest !== "undefined") {
		var url = 'overview_ship.php';
		GM_xmlhttpRequest({
			method: "GET",
			url: url,
			onload: function(response) {
				GM_setValue(getUniverse() + '_Armor_Strength', response.responseText.match(/Strength x(\d)/i)[1]);
			}
		});
	}
};

function getArmorStrength() {
	var armStr = GM_getValue(getUniverse() + '_Armor_Strength', '0');
	if (armStr === '0') {
		armStr = 1;
		setArmorStrength();
	}
	return armStr;
};

function determineShipArmor(shipName) {
	var shipArmor = {
		'tyrant':60, 'junkeriv':90, 'rustclaw':135, 'elpadre':195, 'hercules':195, 
		'babel_transporter':210, 'harrier':210, 'rustfire':210, 'sabre':225, 
		'spectre':225, 'wasp':240, 'adder':255, 'ficon':270, 'thunderbird':270, 
		'constrictor':285, 'interceptor':285, 'slider':285, 'lanner_mini':300, 
		'leviathan':330, 'marauder':330, 'trident':330, 'extender':345, 'celeus':360, 
		'piranha':375, 'venom':375, 'blood_lanner':390, 'lanner':390, 'mantis':390, 
		'mercury':390, 'behemoth':405, 'shadow_stealth_craft':405, 
		'viper_defence_craft':405, 'rover':420, 'vulcan':425, 'gargantua':450, 
		'boa_ultimate_carrier':465, 'chitin':465, 'hawk':480, 'horpor':480, 
		'pantagruel':480, 'reaper':495, 'nano':500, 'dominator':510, 'liberator':510, 
		'nighthawk':510, 'phantom_advanced_stealth_craft':525, 'liberator_eps':540, 
		'mooncrusher':540, 'nighthawk_deluxe':540, 'gauntlet':585, 'sudden_death':600, 
		'scorpion':630, 'doomstar':720, 'war_nova':990, 'harvester':705
	};
	return shipArmor[shipName];
};

function determineRobotsAvailable() {
	var els = document.querySelectorAll('input[name="resid"]');
	for (var loop=0; loop<els.length; loop++) {
		if (els[loop].value == '8') {
			return els[loop].parentNode.previousSibling.textContent;
		}
	}
	return 0;
};

function determineRobotsRequired() {
	var shipName = document.querySelector('table[width="50%"] img').src.match(/ships\/(.+)(_paint0\d)*\.png/i)[1];
	var maxArmor = determineShipArmor(shipName);
	var curArmor = document.querySelector('table[width="50%"] table').rows[1].innerHTML.match(/Armor points: (\d+)/i)[1];
	if (curArmor < maxArmor) {
		var deltaArmor = maxArmor - curArmor;
		var strengthArmor = getArmorStrength();
		var repairType = getRobotRepairType();
		if (repairType === 0) {
			return Math.floor(deltaArmor / (180 / strengthArmor));
		} else
		if (repairType === 1) {
			return Math.ceil(deltaArmor / (180 / strengthArmor));
		} else {
			deltaArmor = getRobotRepairMinimum() - curArmor;
			if (deltaArmor > 0) {
				return Math.ceil(deltaArmor / (180 / strengthArmor));
			}
		}
	}
	return 0;
};

function populateRobotUseInput() {
	var els = document.querySelectorAll('input[name="resid"]');
	for (var loop=0; loop<els.length; loop++) {
		if (els[loop].value == '8') {
			var avaRobots = determineRobotsAvailable();
			if (avaRobots) {
				var reqRobots = determineRobotsRequired();
				if (reqRobots) els[loop].nextSibling.value = avaRobots < reqRobots ? avaRobots : reqRobots;
			}
		}
	}
};

function handleKeyPress(e) {
	if (window.name == '' || e.ctrlKey || e.metaKey || e.altKey || e.target.nodeName == 'INPUT' || e.target.nodeName == 'TEXTAREA') {
		return;
	}
	if (String.fromCharCode(e.which).toLowerCase() == getRobotUseShortcut().toLowerCase()) {
		var els = document.querySelectorAll('input[name="useres"]');
		for (var loop=0; loop<els.length; loop++) {
			if (els[loop].previousSibling.previousSibling.previousSibling.value == '8') {
				els[loop].click();
			}
		}
	}
};

/*****************************************************************************************
	Script options settings table
*****************************************************************************************/

function showOptions() {
	var table = document.createElement('table');
	table.setAttribute('class', 'messagestyle');
	table.style.backgroundImage = document.querySelector('table.messagestyle').style.backgroundImage;
	table.style.position = 'absolute';
	table.style.top = '20px';
	table.style.right = '20px';
	var html = '';
	html += '<tr><th>NPC Robot Repair Settings</th></tr>';
	html += '<tr><td><label><input type="text" id="use_robots_shortcut_txt" maxlength="1" size="1" style="text-align:center;"/> Use robots shortcut key</label></td></tr>';
	html += '<tr><td><label><input type="radio" name="repair_options_radio" id="roundup_rdo" value="1"/> Round up to full repair</label></td></tr>';
	html += '<tr><td><label><input type="radio" name="repair_options_radio" id="partial_rdo" value="0"/> Do not waste the last robot</label></td></tr>';
	html += '<tr><td><label><input type="radio" name="repair_options_radio" id="roundupmin_rdo" value="2"/> Repair to at least</label> <input type="text" id="roundup_min_txt" maxlength="3" size="3" style="text-align:center;"/></td></tr>';
	html += '<tr><td><button id="repair_btn_save" style="color:#0f0;background-color:#292;border-width:1px;padding:2px;">Save Settings</button></td></tr>';
	table.innerHTML = html;
	document.body.appendChild(table);
	document.getElementById('use_robots_shortcut_txt').value = getRobotUseShortcut();
	switch (getRobotRepairType()) {
		case 0 : document.getElementById('partial_rdo').checked = 'checked'; break;
		case 1 : document.getElementById('roundup_rdo').checked = 'checked'; break;
		case 2 : document.getElementById('roundupmin_rdo').checked = 'checked'; break;
	}
	document.getElementById('roundup_min_txt').value = getRobotRepairMinimum();
	document.getElementById('repair_btn_save').addEventListener('click', saveSettings);
};

function saveSettings() {
	GM_setValue(getUniverse() + '_Shortcut_Key', document.getElementById('use_robots_shortcut_txt').value);
	if (document.getElementById('partial_rdo').checked) {
		GM_setValue(getUniverse() + '_Robot_Repair_Type', '0');
	} else
	if (document.getElementById('roundup_rdo').checked) {
		GM_setValue(getUniverse() + '_Robot_Repair_Type', '1');
	} else {
		GM_setValue(getUniverse() + '_Robot_Repair_Type', '2');
	}
	GM_setValue(getUniverse() + '_Robot_Repair_Min', document.getElementById('roundup_min_txt').value);
	var loc = location.href;
	location.href = loc;
};

/*****************************************************************************************
	Set the armor strength every visit to the ship overview page
*****************************************************************************************/

if (document.URL.indexOf('pardus.at/overview_ship.php') > -1) {
	GM_setValue(getUniverse() + '_Armor_Strength', document.body.innerHTML.match(/Strength x(\d)/i)[1]);
}

/*****************************************************************************************
	Auto-populate the number of robots to repair when in NPC combat
*****************************************************************************************/

else {
	showOptions();
	populateRobotUseInput();
	window.addEventListener('keydown', handleKeyPress, true);
}